package redis;

import com.sun.jna.*;
import com.sun.jna.ptr.*;
import org.apache.commons.lang3.time.StopWatch;
import org.opencv.core.CvType;
import org.opencv.core.Mat;
import org.opencv.imgproc.Imgproc;
import redis.clients.jedis.Jedis;
import util.JedisUtil;
import util.Mat2BufImg;
import util.ThreadPoolUtil;

import java.awt.image.BufferedImage;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;

public class SendToRedis {
    private String ip;
    private String account = "admin";
    private String password = "password";
    private Long count = 1L;
    private boolean loginOk = false;
    private boolean realPlayOk = false;
    private Object locker = new Object();
    private String logHead;
    private SimpleDateFormat sdfSS = new SimpleDateFormat("HH:mm:ss");
    private volatile StopWatch stopWatch = new StopWatch();
    private Mat src;
    private Mat dst;
    private BufferedImage bufferedImage;

    static HCNetSDK hCNetSDK = HCNetSDK.INSTANCE;
    static PlayCtrl playControl = PlayCtrl.INSTANCE;
    HCNetSDK.NET_DVR_DEVICEINFO_V30 m_strDeviceInfo;//设备信息
    HCNetSDK.NET_DVR_CLIENTINFO m_strClientInfo;//用户参数
    NativeLong lUserID;//用户句柄
    NativeLong lPreviewHandle;//预览句柄
    NativeLongByReference m_lPort;//回调预览时播放库端口指针
    FRealDataCallBack fRealDataCallBack;//预览回调函数实现
    FDecCallBack fDecCallBack;//解码回调函数实现
    String m_sDeviceIP;//已登录设备的IP地址

    public SendToRedis(String ip) {
        this.ip = ip;
        lUserID = new NativeLong(-1);
        lPreviewHandle = new NativeLong(-1);
        m_lPort = new NativeLongByReference(new NativeLong(-1));
        fRealDataCallBack = new FRealDataCallBack();
        fDecCallBack = new FDecCallBack();
        logHead = "----------> " + ip + " ";
    }

    public void run() {
        // clear();
        // login(ip, 8000, account, password);
        // realPlay();

        Timer timer = new Timer();
        timer.scheduleAtFixedRate(new TimerTask() {
            @Override
            public void run() {
                synchronized (locker) {
                    if (!loginOk || !realPlayOk) {
                        Jedis jedis = JedisUtil.getJedis();
                        jedis.set(ip, "no");
                        JedisUtil.close(jedis);
                        clear();
                        login(ip, 8000, account, password);
                        realPlay();
                        while (loginOk && realPlayOk) {
                            // int a = 1;
                        }
                    }
                }
            }
        }, 1000, 1000 * 60);
    }

    public void login(String ip, int port, String userName, String password) {
        if (hCNetSDK.NET_DVR_Init()) {
            System.out.println(logHead + "初始化成功 ");
        }
        if (lUserID.longValue() > -1) {
            hCNetSDK.NET_DVR_Logout_V30(lUserID);
            lUserID = new NativeLong(-1);
        }
        if (lPreviewHandle.longValue() > -1) {
            hCNetSDK.NET_DVR_StopRealPlay(lPreviewHandle);
        }
        //注册
        m_sDeviceIP = ip;//设备ip地址
        m_strDeviceInfo = new HCNetSDK.NET_DVR_DEVICEINFO_V30();

        lUserID = hCNetSDK.NET_DVR_Login_V30(m_sDeviceIP, (short) port, userName, password, m_strDeviceInfo);
        System.out.println(logHead + "byChanNum " + m_strDeviceInfo.byChanNum + "  byStartChan " + m_strDeviceInfo.byStartChan);
        long userID = lUserID.longValue();
        if (userID == -1) {
            m_sDeviceIP = "";//登录未成功,IP置为空
            System.out.println(logHead + "注册失败");
            loginOk = false;
        } else {
            System.out.println(logHead + "注册成功");
            loginOk = true;
        }
    }

    public void realPlay() {
        m_strClientInfo = new HCNetSDK.NET_DVR_CLIENTINFO();
        m_strClientInfo.lChannel = new NativeLong(m_strDeviceInfo.byStartChan);
        m_strClientInfo.hPlayWnd = null;
        lPreviewHandle = hCNetSDK.NET_DVR_RealPlay_V30(lUserID, m_strClientInfo, fRealDataCallBack, null, true);
        long previewSucValue = lPreviewHandle.longValue();
        if (previewSucValue == -1) {
            System.out.println(logHead + "预览失败");
            realPlayOk = false;
        } else {
            System.out.println(logHead + "预览成功");
            realPlayOk = true;
        }
    }

    /******************************************************************************
     * 内部类:   FRealDataCallBack
     * 实现预览回调数据
     ******************************************************************************/
    class FRealDataCallBack implements HCNetSDK.FRealDataCallBack_V30 {
        public void invoke(NativeLong lRealHandle, int dwDataType, ByteByReference pBuffer, int dwBufSize, Pointer pUser) {
            switch (dwDataType) {
                case HCNetSDK.NET_DVR_SYSHEAD: //系统头
                    System.out.println(logHead + "FRealDataCallBack 系统头");
                    if (!playControl.PlayM4_GetPort(m_lPort)) //获取播放库未使用的通道号
                    {
                        break;
                    }
                    if (dwBufSize > 0) {
                        if (!playControl.PlayM4_SetDecCallBack(m_lPort.getValue(), fDecCallBack))  //设置解码回调
                        {
                            break;
                        }
                        // 设置解码回调函数 解码且显示
                        // if (!playControl.PlayM4_SetDecCallBackEx(m_lPort.getValue(), fDecCallBack, null, null)) {
                        //     break;
                        // }
                        if (!playControl.PlayM4_SetStreamOpenMode(m_lPort.getValue(), PlayCtrl.STREAME_REALTIME))  //设置实时流播放模式
                        {
                            break;
                        }

                        if (!playControl.PlayM4_OpenStream(m_lPort.getValue(), pBuffer, dwBufSize, 1024 * 1024)) //打开流接口
                        {
                            break;
                        }
                        if (!playControl.PlayM4_Play(m_lPort.getValue(), null)) //播放开始
                        {
                            break;
                        }
                    }
                case HCNetSDK.NET_DVR_STREAMDATA:   //码流数据
                    if (!playControl.PlayM4_InputData(m_lPort.getValue(), pBuffer, dwBufSize)) {
                        break;
                    }
            }
        }
    }

    class FDecCallBack implements PlayCtrl.DecCallBack {
        @Override
        public void invoke(NativeLong nPort, ByteByReference pBuffer, NativeLong nSize, PlayCtrl.FRAME_INFO frameInfo, NativeLong nReserved1, NativeLong nReserved2) {
            if (++count % 6 == 0) {
                try {
                    handle(pBuffer, nSize.intValue(), frameInfo);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }

        //这样在回调函数DecCallBack 中可以得到视音频数据，其中视频数据是YV12格式的，音频数据是PCM格式的。
        public void handle(ByteByReference pBuffer, int dwBufSize, PlayCtrl.FRAME_INFO frameInfo) {
            src = null;
            dst = null;
            bufferedImage = null;
            boolean isLog = new Date().getSeconds() % 10 == 0;
            String printStr = null;
            if (isLog) {
                stopWatch.reset();
                stopWatch.start();
            }

            int width = frameInfo.nWidth.intValue();
            int height = frameInfo.nHeight.intValue();
            byte[] byteArray = pBuffer.getPointer().getByteArray(0, dwBufSize);

            src = new Mat(height + height / 2, width, CvType.CV_8UC1);
            src.put(0, 0, byteArray);
            if (isLog) {
                stopWatch.split();
                printStr = "";
                printStr += logHead + "put " + stopWatch.getSplitTime() + "\r\n";
            }

            dst = new Mat(height, width, CvType.CV_8UC3);
            Imgproc.cvtColor(src, dst, Imgproc.COLOR_YUV2BGR_YV12);
            if (isLog) {
                stopWatch.split();
                printStr += logHead + "cvtColor " + stopWatch.getSplitTime() + "\r\n";
            }

            bufferedImage = Mat2BufImg.Mat2BufImg(dst);
            if (isLog) {
                stopWatch.split();
                printStr += logHead + "Mat2BufImg " + stopWatch.getSplitTime() + "\r\n";
            }

            String imgHex = Mat2BufImg.bufImgToBase64(bufferedImage, "jpg");
            if (isLog) {
                stopWatch.split();
                printStr += logHead + "bufImgToBase64 " + stopWatch.getSplitTime() + "\r\n";
            }

            Jedis jedis = JedisUtil.getJedis();
            jedis.set(ip, imgHex);
            JedisUtil.close(jedis);
            if (isLog) {
                stopWatch.split();
                printStr += logHead + "set " + stopWatch.getSplitTime() + "\r\n";
                System.out.println(printStr);
            }
        }
    }

    public void clear() {
        //如果在预览,先停止预览, 释放句柄
        if (lPreviewHandle.longValue() > -1) {
            hCNetSDK.NET_DVR_StopRealPlay(lPreviewHandle);
        }
        //如果已经注册,注销
        if (lUserID.longValue() > -1) {
            hCNetSDK.NET_DVR_Logout_V30(lUserID);
        }
        //cleanup SDK
        hCNetSDK.NET_DVR_Cleanup();
    }
}
